/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.audiocd;

import javax.swing.JPanel;

import net.sf.dc.console.InternetSearchForm;
import net.sf.dc.console.ItemForm;
import net.sf.dc.console.MainFrame;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModules;
import net.sf.dc.processes.http.FreeDBAudioCDUpdate;
import net.sf.dc.resources.DCLabels;
import net.sf.ext.jflexsett.settings.Settings;
import net.sf.ext.util.Resources;

/**
 * Dialog for retrieving information from an audio CD in the CD-Rom drive.
 * Uses FreeDB to get the information based on the discid
 * @see net.sf.dc.freedb.Freedb
 * @see net.sf.dc.processes.http.FreeDBAudioCDUpdate
 * 
 * @author Robert-Jan van der Waals
 * @since 1.4
 * @version 1.50
 */
public class ImportAudioCDDialog extends InternetSearchForm {
	
    public ImportAudioCDDialog(ItemForm itemForm) {
        super(itemForm, null, itemForm);
        setTitle(Resources.getText(DCLabels.lblImportAudioCD));
        setHelpIndex("dc.Inserting.Odb");
    }
    
    public ImportAudioCDDialog(Class clazz, int rowNumber) {
        super(DataCrow.mainFrame, null, MainFrame._INSERTTAB);
        setTitle(Resources.getText(DCLabels.lblImportAudioCD));
        setHelpIndex("dc.Inserting.Odb");
    }
    
    public JPanel getSearchPanel() {
        return null;
    }

    public void setCancelMode() {
        cancelled = false;
        buttonStopSearch.setEnabled(true);
    }    
    
    public void setSearchMode() {
        buttonStopSearch.setEnabled(false);        
    }
    
    /**
     * Disc ID search
     */
    public void startSearch() {
        saveSettings();
        setCancelMode();
        process = new FreeDBAudioCDUpdate(this, DCModules.getModule(DCModules._AUDIOCD));
        process.setPriority(Thread.MAX_PRIORITY);
        process.start();
    }    
    
    /**
     * Initialize the values for this dialog
     */
    protected void setDefaultValues() {
        setSize(Settings.getValueAsDimension(
                Repository.settings.stFreeDBSearchFormViewSize));          
    }
}
