/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.components.renderers;

import java.awt.Component;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.ListCellRenderer;

import net.sf.dc.core.IconLibrary;

/**
 * Used to show images and text in a combo box
 * 
 * @author Robert Jan van der Waals
 * @since 1.3
 * @version 1.00
 */
public class RatingComboBoxRenderer extends JLabel implements ListCellRenderer {
    
    public RatingComboBoxRenderer() {
        setOpaque(true);
        setHorizontalAlignment(LEFT);
        setVerticalAlignment(CENTER);
    }

    public Component getListCellRendererComponent(
                                       JList list,
                                       Object value,
                                       int index,
                                       boolean isSelected,
                                       boolean cellHasFocus) {

        ImageIcon icon = null;
        setText("");
        
        if (!(value instanceof Integer) || value == null || value.equals(new Integer(0))) {
            setIcon(IconLibrary._icoRating0);
        } else {
            int rating = ((Integer) value).intValue();
            switch (rating) {
                case 1:
                    icon = IconLibrary._icoRating1;
                    break;
                case 2:
                    icon = IconLibrary._icoRating2;
                    break;
                case 3:
                    icon = IconLibrary._icoRating3;
                    break;
                case 4:
                    icon = IconLibrary._icoRating4;
                    break;                    
                case 5:
                    icon = IconLibrary._icoRating5;
                    break;                    
            }
            setIcon(icon);
        }
        
        return this;
    }
}
