/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.dialogs;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ScrollPaneConstants;

import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.components.Layout;
import net.sf.dc.core.Repository;
import net.sf.dc.resources.DCLabels;
import net.sf.ext.jflexsett.settings.Settings;
import net.sf.ext.util.Resources;
import net.sf.ext.util.SwingUtil;

/**
 * Provides the user with a large, very basic, text editor.
 * This view is always on top (modal)
 * 
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version 1.00
 */
public class TextDialog extends JDialog {
    
    private JTextArea textArea = ComponentFactory.getTextArea();
    private JPanel panelText = new JPanel(false);
    private JPanel panelAction = new JPanel(false);
    private Color color = Settings.getValueAsColor(Repository.settings.stColor);
    
    /**
     * Creates the text view and shows it. The view is always on top
     * @param s the text to be displayed
     */
    public TextDialog(String s) {
        super(SwingUtil.getRootFrame());
        this.getContentPane().setBackground(color);
        buildPanel();
        
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                Settings.setValue(Repository.settings.stTextViewerSize,
                                  getSize());
                dispose();
            }
        });
        
        textArea.setText(s);
        setSize(Settings.getValueAsDimension(
                Repository.settings.stTextViewerSize));
        setModal(true);
        setVisible(true);
    }
    
    public String getText() {
        return textArea.getText();
    }
    
    private void buildPanel() {
        panelText.setBackground(color);
        panelAction.setBackground(color);

        //**********************************************************
        //Text panel
        //**********************************************************
        panelText.setLayout(Layout.getGBL());

        JScrollPane pane = new JScrollPane(textArea);
        pane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        pane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);

        panelText.add(pane, Layout.getGBC( 0, 0, 1, 1, 40.0, 40.0
                     ,GridBagConstraints.NORTH, GridBagConstraints.BOTH,
                      new Insets( 5, 5, 5, 5), 0, 0));

        //**********************************************************
        //Action panel
        //**********************************************************
        panelAction.setLayout(Layout.getGBL());
        JButton buttonClose = ComponentFactory.getButton(
                                Resources.getText(DCLabels.lblClose));
        buttonClose.addActionListener(new CloseAction());
        panelAction.add(buttonClose, Layout.getGBC( 0, 0, 1, 1, 0.0, 0.0
                        ,GridBagConstraints.NORTH, GridBagConstraints.NONE,
						new Insets( 5, 5, 5, 5), 0, 0));
        
        //**********************************************************
        //Main panel
        //**********************************************************
        this.getContentPane().setLayout(Layout.getGBL());
        setTitle(Resources.getText(DCLabels.lblTextViewer));
        
        this.getContentPane().add(panelText,  Layout.getGBC( 0, 0, 5, 1, 1.0, 1.0
                                 ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                  new Insets( 0, 0, 0, 0), 0, 0));
        this.getContentPane().add(panelAction,Layout.getGBC( 0, 1, 1, 1, 0.0, 0.0
                                 ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                  new Insets( 0, 0, 0, 0), 0, 0));
    }    
    
    private class CloseAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            dispose();
        }
    }    
}
