/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.movie;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JTextField;

import net.sf.dc.console.InternetSearchForm;
import net.sf.dc.console.ItemForm;
import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.components.Layout;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.core.objects.movie.Movie;
import net.sf.dc.processes.http.IMDBMovieUpdate;
import net.sf.dc.resources.DCLabels;
import net.sf.dc.resources.DCMessages;
import net.sf.ext.jflexsett.settings.Settings;
import net.sf.ext.util.Resources;
import net.sf.ext.util.Utilities;

/**
 * Form used to search for IMDB data. The search is performed on the
 * title of the movie
 *
 * @see net.sf.dc.console.SearchForm
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version 0.50
 */
public class ImdbMovieSearchForm extends InternetSearchForm {
    
    protected JTextField textSearch;
    protected JButton buttonSearch;
    
    /**
     * Create the form
     * @param dco Object to be used to base the search on
     * @param itemForm item form from which the search was started
     */
    public ImdbMovieSearchForm(DCObject dco, ItemForm itemForm) {
        super(itemForm, dco, itemForm);
        setTitle(Resources.getText(DCLabels.lblImdbUpdateProcess));
    }
    
    /**
     * Create the form
     * @param dco Object to be used to base the search on
     * @param clazz class to which the output is targeted
     * @param rowNumber table row number, representing the to be updated object
     */
    public ImdbMovieSearchForm(DCObject dco, int tab, int rowNumber) {
        super(DataCrow.mainFrame, dco, tab);
        setTitle(Resources.getText(DCLabels.lblImdbUpdateProcess));
    }
    
    public void setDefaultValues() {
        setSize(Settings.getValueAsDimension(
                Repository.settings.stImdbSearchFormViewSize)); 
        
        if (dco != null) {
        	textSearch.setText(dco.getValue(Movie._A_TITLE).toString());
        }
        
        Dimension dim = Utilities.getCenteredWindowLocation(getSize());
        setLocation(dim.width, dim.height);        
    }

    public void setCancelMode() {
        cancelled = false;
        buttonSearch.setEnabled(false);
        buttonStopSearch.setEnabled(true);
    }    
    
    public void setSearchMode() {
        buttonSearch.setEnabled(true);
        buttonStopSearch.setEnabled(false);        
    }
    
    public void saveSettings() {
        super.saveSettings();
        Settings.setValue(Repository.settings.stImdbSearchFormViewSize,
                          getSize());
    }   
    
    /**
     * Sends queries to the internet server and retrieves the information
     */
    public void startSearch() {
        setCancelMode();
        saveSettings();
        
        String keywords = textSearch.getText();
        if (!keywords.equals("")) {
            process  = new IMDBMovieUpdate(this, textSearch.getText(), checkBroadband.isSelected());
            process.setPriority(Thread.MAX_PRIORITY);
            process.start();
        } else {
            setSearchMode();
            new MessageBox(this,
                           Resources.getText(DCMessages.msgIMDBNeedsTitle),
                           MessageBox._WARNING);
        }        
    }
    
    public JPanel getSearchPanel() {
        JPanel panel = new JPanel(false);
        panel.setLayout(Layout.getGBL());
        panel.setBackground(color);
        
        textSearch = ComponentFactory.getTextField();
        buttonSearch = ComponentFactory.getButton(Resources.getText(DCLabels.lblFind));
        buttonSearch.addActionListener(new SearchAction());
        
        panel.setLayout(Layout.getGBL());                                
        panel.add(textSearch,   Layout.getGBC( 0, 0, 1, 1, 90.0, 90.0
                                     ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
                                      new Insets(5, 5, 5, 5), 0, 0));                         
        panel.add(buttonSearch, Layout.getGBC( 1, 0, 1, 1, 1.0, 1.0
                                     ,GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL,
                                      new Insets(5, 5, 5, 5), 0, 0));
        return panel;
    }
    
    private class SearchAction implements ActionListener{
        public void actionPerformed(ActionEvent e) {
            startSearch();   
        }
    }     
}
