/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.note;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.RandomAccessFile;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ScrollPaneConstants;

import net.sf.dc.console.DCPanel;
import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.components.Layout;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.Repository;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DCLabels;
import net.sf.dc.resources.DCMessages;
import net.sf.ext.util.Resources;

/**
 * Panel for writing notes and comments. The notes are written to
 * a plain textfile.
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version 0.50
 */
public class NotePanel extends DCPanel {

    private JButton  buttonSave;
    private JButton buttonClear;
    private JButton buttonLoad;
    private JTextArea textNote;

    private File                    file = new File(Repository.files.flCommentFile);
    private RandomAccessFile        access;

    /**
     * Retrieves the saved note (as a textfile) and display it
     */
    public void loadNote() {
        setHelpIndex("dc.BlockNote");
        try {
            access = new RandomAccessFile(file, "rw");
            long filePointer = 0;
            long fileLength = file.length();

            StringBuffer sb = new StringBuffer("");
            while (filePointer < fileLength) {
                sb.append(access.readLine().trim() + '\n');
                filePointer = access.getFilePointer();
            }

            textNote.setText(sb.toString());
        } catch (Exception exp) {}
    }

    /**
     * Creates this panel
     */
    public NotePanel() {
        super(null, null);
        buildPanel();
        loadNote();
    }
    
    public void clear() {
        textNote.setText("");     
    }

    /**
     * Saves the notes to a text file
     */
    public void saveNote() {
        boolean success = false;
        try {
            access = new RandomAccessFile(file, "rw");
            textNote.getText();
            access.setLength(0l);
            access.writeBytes(textNote.getText());
            access.close();
            success = true;
        } catch (Exception exp) {
            Messages.add(exp);
        } finally {
            if (success) {
                setStatus(Resources.getText(DCMessages.msgSaveHasBeenExecuted));
                new MessageBox(Resources.getText(DCMessages.msgDataSaved), 
                               MessageBox._INFORMATION);
            }
        }
    }

    /**
     * Builds the panel
     */
    private void buildPanel() {
        JPanel panelStatus = getStatusPanel();

        //**********************************************************
        //Note panel
        //**********************************************************
        JPanel panelNote   = new JPanel(false);
        panelNote.setLayout(Layout.getGBL());

        textNote = ComponentFactory.getTextArea();
        textNote.setBackground(ComponentFactory.getTableColor());

        JScrollPane scrollIn = new JScrollPane(textNote);
        scrollIn.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        scrollIn.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);

        panelNote.add(    scrollIn,      Layout.getGBC( 0, 0, 1, 1, 80.0, 80.0
                                        ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                                         new Insets(5, 5, 5, 5), 0, 0));

        //**********************************************************
        //Action panel
        //**********************************************************
        JPanel panelActions= new JPanel(false);
        buttonSave  = ComponentFactory.getButton(Resources.getText(DCLabels.lblSaveChanges));
        buttonClear = ComponentFactory.getButton(Resources.getText(DCLabels.lblClear));
        buttonLoad  = ComponentFactory.getButton(Resources.getText(DCLabels.lblLoad));

        buttonLoad.setMnemonic(KeyEvent.VK_L);
        buttonClear.setMnemonic(KeyEvent.VK_C);
        buttonSave.setMnemonic(KeyEvent.VK_S);

        buttonSave.addActionListener(new SaveButtonAction());
        buttonClear.addActionListener(new ClearButtonAction());
        buttonLoad.addActionListener(new LoadButtonAction());

        panelActions.add(   buttonSave, Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                            new Insets(0,  5, 0, 0), 0, 0));
        panelActions.add(      buttonLoad, Layout.getGBC( 1, 0, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                            new Insets(0,  5, 0, 0), 0, 0));
        panelActions.add(   buttonClear,Layout.getGBC( 2, 0, 1, 1, 1.0, 1.0
                           ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                            new Insets(0,  5, 0, 0), 0, 0));

        //**********************************************************
        //Main panel
        //**********************************************************
        setLayout(Layout.getGBL());

        add( panelNote,    Layout.getGBC( 0, 0, 7, 1, 120.0, 120.0
                          ,GridBagConstraints.NORTH, GridBagConstraints.BOTH,
                           new Insets(5, 5, 5, 5), 0, 0));
        add( panelActions, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                          ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                           new Insets(5, 5, 5, 5), 0, 0));
        add( panelStatus,  Layout.getGBC( 0, 2, 7, 1, 1.0, 1.0
                          ,GridBagConstraints.SOUTH, GridBagConstraints.HORIZONTAL,
                           new Insets(0, 5, 0, 5), 0, 0));
    }

    private class SaveButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            saveNote();
        }
    }

    private class ClearButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            clear();
        }
    }

    private class LoadButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            loadNote();
        }
    }
}
