/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.objects.software;

import javax.swing.ImageIcon;

import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModules;
import net.sf.dc.core.objects.DCFieldDefinition;
import net.sf.dc.core.objects.DCObject;
import net.sf.ext.jflexsett.settings.Settings;

/**
 * Representation of a Software item. Allows saving, updating and querying
 * of Software items
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version 0.50
 */
public class Software extends DCObject {

    public static final int _A_NAME = 1;
    public static final int _B_DESCRIPTION = 2;
    public static final int _C_PUBLISHER  = 3;
    public static final int _D_PLATFORM = 4;
    public static final int _E_RELEASEYEAR = 5;
    public static final int _F_WEBPAGE = 6;
    public static final int _G_FILENAME = 7;
    public static final int _H_TYPE = 8;
    public static final int _I_MULTI = 9;
    public static final int _J_PICTUREFRONT = 10;    
    public static final int _K_PICTUREBACK = 11;
    public static final int _L_PICTURECD = 12;
    public static final int _M_SCREENSHOTONE = 13;
    public static final int _N_SCREENSHOTTWO = 14;
    public static final int _O_SCREENSHOTTHREE = 15;
    public static final int _P_SERIALKEY = 16;
    public static final int _Q_LOCATION = 17;
    public static final int _R_RATING = 18;

    public Software() {
       super();
    }
    
    /**
     * Sets the module index / sets the ownership of this
     * data crow object  
     */  
    protected void setModuleIndex() {
        module = DCModules._SOFTWARE;
    }    

    /**
     * Returns the icon for this object
     */      
    public ImageIcon getIcon() {
        return null;    
    }    
    
    /**
     * Initialize all user settings on this data crow object 
     */       
    protected void initializeSettings() {
        DCFieldDefinition[] fields = getFields();
        
        int[] requiredFields = Settings.getValueAsIntegerArray(
                                  Repository.settings.stSoftwareRequiredFields);
                                  
        for (int i = 0; i < requiredFields.length; i++) {
            fields[requiredFields[i]].setRequired(true);
        }
                                  
        int[] hiddenFields = Settings.getValueAsIntegerArray(
                                Repository.settings.stHiddenFieldsSoftware);   

        for (int i = 0; i < hiddenFields.length; i++) {
            fields[hiddenFields[i]].setHidden(true);
        }        
    }
    
    /**
     * Create a UI representation for this object
     */
    public String toString() {
        return getValue(Software._A_NAME).toString();
    }    
}
