/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.db;

import java.sql.Connection;
import java.sql.Statement;

import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.DataCrow;
import net.sf.dc.messages.Messages;
import net.sf.dc.messages.MessagesWriter;
import net.sf.dc.wf.WorkFlow;

/**
 * @author Robert Jan van der Waals
 * @version 1.7
 * @since 1.4
 */
public class DatabaseConversion {
    
    public boolean start(Connection connection) {

        WorkFlow wf = new WorkFlow();
        
        MessagesWriter._windowShow = true;
        MessagesWriter._windowToFront = true;
        
        try {
            Statement stmt = connection.createStatement();

            Messages.add("Starting the conversion", Messages._STANDARD);
            
            Messages.add("Creating checkpoint (backup) for the database", Messages._STANDARD);
            stmt.execute("CHECKPOINT");
            try {
                stmt.execute("DROP table audiotracks");
                stmt.execute("DROP table movies");
                stmt.execute("DROP table audiocds");
                stmt.execute("DROP table musicfiles");
            } catch (Exception ignore) {}

            Messages.add("Changing the script format to text to allow future database upgrades", Messages._STANDARD);
            stmt.execute("SET SCRIPTFORMAT TEXT;");
            Messages.add("Creating new checkpoint (backup) for the database", Messages._STANDARD);
            stmt.execute("CHECKPOINT");
            
            Messages.add("Backing up table Movie (renamed to SOFT)", Messages._STANDARD);
            stmt.execute("ALTER table Software rename TO SOFT;");
            Messages.add("Backing up table Movie (renamed to MOV)", Messages._STANDARD);
            stmt.execute("ALTER table Movie rename TO MOV;");
            Messages.add("Backing up table AudioCD (renamed to AC)", Messages._STANDARD);
            stmt.execute("ALTER table AudioCD rename TO AC;");
            Messages.add("Backing up table AudioTrack (renamed to ATR)", Messages._STANDARD);
            stmt.execute("ALTER table AudioTrack rename TO ATR;");
            Messages.add("Copying data from SoftwareType to new table softwarecategories", Messages._STANDARD);
            stmt.execute("select * into CACHED softwarecategories FROM SoftwareType;");
            Messages.add("Copying data from SOFT to new table software", Messages._STANDARD);
            stmt.execute("select * into CACHED software FROM SOFT;");
            Messages.add("Copying data from MOV to new table movies", Messages._STANDARD);
            stmt.execute("select * into CACHED movies FROM MOV;");
            Messages.add("Copying data from AC to new table audiocds", Messages._STANDARD);
            stmt.execute("select * into CACHED audiocds FROM AC;");
            Messages.add("Copying data from MP3 to new table musicfiles", Messages._STANDARD);
            stmt.execute("select * into CACHED musicfiles FROM MP3;");
            Messages.add("Copying data from ATR to new table audiotracks", Messages._STANDARD);
            stmt.execute("select * into CACHED audiotracks FROM ATR;");
            
            try {
                Messages.add("Deleting table SoftwareType", Messages._STANDARD);
                stmt.execute("DROP table SoftwareType;");
                Messages.add("Deleting table Mp3", Messages._STANDARD);
                stmt.execute("DROP table Mp3;");
                Messages.add("Deleting table SOFT", Messages._STANDARD);
                stmt.execute("DROP table SOFT;");
                Messages.add("Deleting table MOV", Messages._STANDARD);
                stmt.execute("DROP table MOV;");
                Messages.add("Deleting table ATR", Messages._STANDARD);
                stmt.execute("DROP table ATR;");
                Messages.add("Deleting table AC", Messages._STANDARD);
                stmt.execute("DROP table AC;");
            } catch (Exception ignore) {}   
            
            Messages.add("All Done!", Messages._STANDARD);
            
            DataCrow.closeSplashScreen();
            
            new MessageBox("Conversion was a big success! The database and the application will now be closed.", MessageBox._INFORMATION);
            stmt.execute("SHUTDOWN COMPACT;");
            System.exit(0);
        } catch (Exception exp) {
            Messages.add(exp);
        }
        return false;
    }
}
