/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

/*
 *  Originally this code came from (but has been heavily modified) :
 * 
 *  ********************************************************************   **
 *  Copyright notice                                                       **
 *  **																	   **
 *  (c) 2003 Entagged Developpement Team				                   **
 *  http://www.sourceforge.net/projects/entagged                           **
 *  **																	   **
 *  All rights reserved                                                    **
 *  **																	   **
 *  This script is part of the Entagged project. The Entagged 			   **
 *  project is free software; you can redistribute it and/or modify        **
 *  it under the terms of the GNU General Public License as published by   **
 *  the Free Software Foundation; either version 2 of the License, or      **
 *  (at your option) any later version.                                    **
 *  **																	   **
 *  The GNU General Public License can be found at                         **
 *  http://www.gnu.org/copyleft/gpl.html.                                  **
 *  **																	   **
 *  This copyright notice MUST APPEAR in all copies of the file!           **
 *  ********************************************************************
 */
package net.sf.dc.freedb;

import java.io.IOException;
import java.io.PrintWriter;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.StringTokenizer;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import net.sf.dc.console.audiocd.ImportAudioCDDialog;
import net.sf.dc.core.objects.audiocd.AudioCD;
import net.sf.dc.core.objects.audiocd.AudioTrack;
import net.sf.dc.http.HttpConnection;
import net.sf.dc.resources.DCMessages;
import net.sf.ext.util.Resources;

public class Freedb {
    
    private final ImportAudioCDDialog ui;
    //private String server = "de.freedb.org";
    private final String server;
    private String userLogin = formatProperty(System.getProperty("user.name"));
    private String userDomain = formatProperty(System.getProperty("os.name"));
    private String clientName = "DataCrow";
    private String clientVersion = "1.0";
    private String protocol = "6";

    public Freedb(String server, ImportAudioCDDialog ui) {
        this.ui = ui;
        this.server = server;
    }
    
    public AudioCD[] queryDiscId(String discID) throws Exception {
        AudioCD[] audioCDs = query(discID);
        return audioCDs;
    }    
    
    /**
     * Query the site parse the resulting html, and create appropriate queryresults
     * URL of the search page is like
     * http://www.freedb.org/freedb_search.php? invariant
     * words=bela+fleck   the search terms spaces are escaped 
     *                    (accented letters are not permitted)
     * &allfields=NO      Search in all fields ?
     * &fields=artist     Search in artists
     * &fields=title      Search in albums
     * &allcats=YES       Search in All categories ?
     * &grouping=none     Group the results in a single list or in genres
     * &x=0               ??(not needed)
     * &y=0               ??(not needed)
     * 
     * @param search
     * @return query results
     * @throws Exception
     */
/**    public FreedbQueryResult[] queryKeywords(String search) throws Exception {
        String answer = searchFreedb(search);
        
        StringTokenizer st = new StringTokenizer(answer, "\n");
        LinkedList list = new LinkedList();
        String[] infos = new String[4];
        
        int counter = 0;
        ui.initProgressBar(st.countTokens());
        while (st.hasMoreTokens()) {
            ui.updateProgressBar(counter);
            String line = st.nextToken();
            if (bigMatch(line, infos)) { 
                // We have a line that has an artist/album
                list.add(new FreedbQueryResult(infos[0]+" "+infos[1]+" "+infos[2]));
                if (smallMatch(infos[3], infos)) {
                    // Do we have a number 2 of the same album/artist ?
                    list.add(new FreedbQueryResult(infos[0]+" "+infos[1]+" "+infos[2]));
                }
            } else if(smallMatch(line, infos)) {
                // We have a line that is number 3,4 of the previous artist/album
                list.add(new FreedbQueryResult(infos[0]+" "+infos[1]+" "+infos[2]));
            }
            counter++;
        }
        
        FreedbQueryResult[] results = new FreedbQueryResult[list.size()];
        Iterator it = list.iterator();
        int i = 0;
        while (it.hasNext()) {
            results[i] = (FreedbQueryResult) it.next();
            i++;
        }
        
        return results;
    }  */   
    
    public String[] getCategories() throws Exception {
        String answer = askFreedb("cddb+lscat");
        
        StringTokenizer st = new StringTokenizer(answer, "\n");
        
        List l = new LinkedList();
        while(st.hasMoreTokens()) {
            String line = st.nextToken();
            if(!line.startsWith("2") && !line.startsWith(".") ) {
                l.add(line);
            }
        }
        
        String[] categories = new String[l.size()];
        Iterator it = l.iterator();
        int i = 0;
        while(it.hasNext()) {
            categories[i] = (String) it.next();
            i++;
        }
        return categories;
    }
    
    
    public String[] getAvailableServers() throws Exception {
        String answer = askFreedb("sites");
        StringTokenizer st = new StringTokenizer(answer, "\n");
        
        List l = new LinkedList();
        while(st.hasMoreTokens()) {
            String line = st.nextToken();
            if(!line.startsWith("2") && !line.startsWith(".") ) {
                String[] sline = line.split(" ", 7);
                if(sline[1].equals("http"))
                    l.add( sline[0] );
            }
        }
        
        String[] servers = new String[l.size()];
        Iterator it = l.iterator();
        int i = 0;
        while(it.hasNext()) {
            servers[i] = (String) it.next();
            i++;
        }
        return servers;
    }

    /**
     * Queries the freedb server for the full id:
     * Client command:
     * -> cddb query discid ntrks off1 off2 ... nsecs
     * 
     * After querying, the cd is queried by its discid and genre
     * 
     * @param id full disc id
     * @throws Exception
     */
    public AudioCD[] query(String id) throws Exception {
        //Create the command to be sent to freedb
        String command = getReadCommand(id);
        //Send the command, and read the answer
        String queryAnswer = askFreedb(command);
        
        StringTokenizer st = new StringTokenizer(queryAnswer, "\n" );
        String[] answers = new String[st.countTokens()];
        for (int i = 0; i < answers.length; i++) {
            answers[i] = st.nextToken();
        }
      
        int counter = 0;
        String[][] data = new String[answers.length][];
        for (int i = 0; i < answers.length; i++) {
        	String answer = answers[i];
            
            if (answer.startsWith("200")) {
            	answer = answer.substring(3, answer.length());
            }
            
            if (!answer.startsWith("211") && !answer.startsWith("210") && !answer.startsWith(".")) {
                st = new StringTokenizer(answer, " " );
                String[] row = new String[st.countTokens()];
                for (int j = 0; j < 2; j++) {
                    row[j] = st.nextToken();
                }
                data[i] = row;
                counter++;
            }
        }
        
        AudioCD[] audioCDs = new AudioCD[counter];
        counter = 0;
        for (int i = 0; i < data.length; i++) {
            if (data[i] != null) {
            	String genre = data[i][0];
            	String discid = data[i][1];
                
                FreedbReadResult result = read(genre, discid);
                audioCDs[counter++] = convertToDcObject(result); 
            }
        }
        return audioCDs;
    }
    
    public FreedbReadResult read(String genre, String id) throws Exception {
        //Create the command to be sent to freedb
        String command = getReadCommand(genre, id);
        //Send the command, and read the answer
        String queryAnswer = askFreedb(command);
        //Parse the result
        return new FreedbReadResult(queryAnswer, genre);
    }
    
    public FreedbReadResult read(FreedbQueryResult query) throws Exception {
        //Create the command to be sent to freedb
        String command = getReadCommand(query);
        ui.addMessage(Resources.getText(DCMessages.msgSendCommandToServer, command));
        //Send the command, and read the answer
        String queryAnswer = askFreedb(command);
        ui.addMessage(Resources.getText(DCMessages.msgGotResult));
        //Parse the result
        return new FreedbReadResult(queryAnswer, query.isExactMatch());
    }    
    
    private String searchFreedb(String search) throws Exception {
        String terms = search.replaceAll(" ", "+");
        
		URL url = null;
		try {
			url = new URL( "http://www.freedb.org/freedb_search.php?words="+terms+
                           "&allfields=NO&fields=artist&fields=title&allcats=YES&grouping=none");
		} catch (MalformedURLException e) {
			throw new Exception(Resources.getText(DCMessages.msgInvalidURL, url.toString()));
		}

        HttpConnection connection = new HttpConnection(url);
		String output = connection.getString(url);;
		return output;
    }
    
    private String askFreedb(String command) throws Exception {
		URL url = null;
		try {
			url = new URL( "http://"+server+":80/~cddb/cddb.cgi" );
		} catch ( MalformedURLException e ) {
			throw new Exception(Resources.getText(DCMessages.msgInvalidURLChangeSetting, url.toString()));
		}
        
        HttpConnection connection = new HttpConnection(url);

		try {
			PrintWriter out = new PrintWriter(connection.getOutputStream());
            String query = "cmd=" + command + "&hello=" + 
                           userLogin + "+" + 
                           userDomain + "+" + 
                           clientName + "+" + 
                           clientVersion + 
                           "&proto="+ protocol;
            out.println(query);
            out.flush();
            out.close();
		} catch ( IOException e ) {
			throw new Exception(Resources.getText(DCMessages.msgFreeDBServerUnreachable, e.getMessage()));
		}
		String output = connection.getString(url);

		//Preliminary freedb error check, error codes 4xx and 5xx indicate an error
		if (output.startsWith("4") || output.startsWith("5")) {
			throw new Exception(Resources.getText(DCMessages.msgFreeDBServerReturnedError, output));
        }
		return output;
	}
    
    private String getReadCommand(FreedbQueryResult query) {
		return "cddb+read+" + query.getCategory() + "+" + query.getDiscId();
	}
	
    private String getReadCommand(String id) {
        return "cddb+query+"+ id;
    }
    
	private String getReadCommand(String genre, String id) {
		return "cddb+read+" + genre + "+" + id;
	}
	
    
    private boolean bigMatch(String line, String[] infos) {
        Matcher matcher = Pattern.compile(".+?php\\?cat=(.+?)&id=(.+?)\">(.+?) \\/ (.+?)<\\/(.*)").matcher(line);
		if (matcher.matches()) {
		    infos[0] = matcher.group(1);
			infos[1] = matcher.group(2);
			infos[2] = matcher.group(3);
			infos[2] += " / "+matcher.group(4);
			infos[3] = matcher.group(5);
		}
		return matcher.matches();
    }
    
    private boolean smallMatch(String line, String[] infos) {
        Matcher matcher = Pattern.compile(".+?php\\?cat=(.+?)&id=(.+?)\">.*<\\/.*").matcher(line);
        if (matcher.matches()) {
		    infos[0] = matcher.group(1);
			infos[1] = matcher.group(2);
		}
		return matcher.matches();
	}
    
    private String formatProperty(String s) {
        if(s.length() == 0)
            return "default";
        int i = s.indexOf(" ");
        return (i == -1) ? s : s.substring(0, i);
    }
    
    /**
     * Converts a query result (not detailed) to a Data Crow Object
     * @param result query result
     */
    public AudioCD convertToDcObject(FreedbQueryResult result) {
        AudioCD audioCD = new AudioCD();
        audioCD.setValue(AudioCD._A_TITLE, result.getAlbum());
        audioCD.setValue(AudioCD._B_ARTIST, result.getArtist());
        audioCD.setValue(AudioCD._D_GENRE, result.getCategory());
        audioCD.setValue(AudioCD._H_DISCID, result.getDiscId());
        return audioCD;
    }
    
    /**
     * Converts a read result (detailed) to a Data Crow Object
     * @param result read result
     */
    public AudioCD convertToDcObject(FreedbReadResult result) {
        AudioCD audioCD = new AudioCD();
        audioCD.setValue(AudioCD._A_TITLE, result.getAlbum());
        audioCD.setValue(AudioCD._B_ARTIST, result.getArtist());
        audioCD.setValue(AudioCD._D_GENRE, result.getCategory());
        audioCD.setValue(AudioCD._H_DISCID, result.getDiscId());
        
        try {
            Integer year = new Integer(Integer.parseInt(result.getYear()));
            audioCD.setValue(AudioCD._C_YEAR, year);
        } catch (NumberFormatException ignore) {}
        
        for (int i = 0; i < result.getTracksNumber(); i++) {
            AudioTrack track = new AudioTrack();
            track.setValue(AudioTrack._A_TRACKNUMBER, new Integer(i + 1));
            track.setValue(AudioTrack._B_TITLE, result.getTrackTitle(i));
            track.setValue(AudioTrack._C_LYRICS, result.getTrackComment(i));
            audioCD.addChild(track);
        }
        return audioCD;
    }    
}
