/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.processes.http;

import java.net.URL;
import java.util.Enumeration;
import java.util.Vector;

import net.sf.dc.console.InternetSearchForm;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.core.objects.software.Software;
import net.sf.dc.http.HttpConnection;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DCMessages;
import net.sf.ext.util.Resources;
import net.sf.ext.util.StringUtils;

/**
 * Searches for software descriptions and screenshots on the Amazon site
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version 0.80
 */
public class AmazonSoftwareUpdate extends InternetUpdateProcess {

    private String searchString;
    private final boolean broadband;
    
    private final InternetSearchForm ui;

    public AmazonSoftwareUpdate(InternetSearchForm ui,
                                String searchString, 
                                boolean broadband) {
        this.ui = ui;
        this.searchString = searchString;
        this.broadband = broadband;
    }
    
    public void run() {
    	search();
    }
    
    /**
     * Searches on the Amazon.com website on the catagories software and
     * videogames. Returns a Vector of Strings. Each String describes 
     * a found item on the Amazon website
     */
    public void search() {
        ui.setCancelMode();
        try {
            ui.addMessage(Resources.getText(DCMessages.msgConnectingToServer, 
                          "xml.amazon.com"));            
            
            searchString = searchString.replaceAll(" ", "%20");
            URL url = new URL("http://xml.amazon.com/onca/xml3?t=webservices-20&" +
                              "dev-t=1N37KF9JXJXVF9JEFB02&KeywordSearch=" + searchString +
                              "&mode=videogames&type=heavy&page=1&f=xml");
            HttpConnection connection = new HttpConnection(url);
            String resultGames = connection.getString(url);
            connection.close();
            
            url = new URL("http://xml.amazon.com/onca/xml3?t=webservices-20&" +
                          "dev-t=1N37KF9JXJXVF9JEFB02&KeywordSearch=" + searchString +
                          "&mode=software&type=heavy&page=1&f=xml");
            connection = new HttpConnection(url);
            String resultSoftware = connection.getString(url);
            String result = resultGames + resultSoftware;
            createObjects(result);
        } catch (Exception e) {
            new MessageBox(e.toString(), MessageBox._ERROR);
            Messages.add(e);
            ui.addMessage("Could not connect to Amazon! " + e.getMessage());
        }
        
        ui.setSearchMode();
    }
    
    /**
     * Splits one search result in to pieces. Every piece (String)
     * is a presentation of one item (raw data)
     * @param rawResult
     */
    private void createObjects(String rawResult) {
        Vector parts = new Vector();
        int index = rawResult.indexOf("<Details");
        while (index > -1 && !cancelled) {
            int end = rawResult.indexOf("</Details>") + 10;
            if (end != -1) {
                String part = rawResult.substring(index, end);
                parts.addElement(part);
                rawResult = rawResult.substring(end, rawResult.length());
            }
            index = rawResult.indexOf("<Details");
        }
        
        if (parts.size() == 0) {
            new MessageBox(Resources.getText(DCMessages.msgNoResultsForKeywords,
                           searchString), MessageBox._WARNING);
            return;
        }
        
        ui.addMessage(Resources.getText(DCMessages.msgFoundXResults, 
                      String.valueOf(parts.size())));          
        ui.addMessage(Resources.getText(DCMessages.msgStartParsingXResults, 
                      String.valueOf(parts.size())));         
        
        ui.initProgressBar(parts.size());
        int counter = 1;
        for (Enumeration enum = parts.elements(); enum.hasMoreElements() && !cancelled; ) {
            DCObject dco = convertXmlToObject(enum.nextElement().toString());
            ui.addMessage(Resources.getText(DCMessages.msgParsingSuccessfull, 
                          dco.getValue(Software._A_NAME).toString()));          
            ui.addObject(dco);
            ui.updateProgressBar(counter++);
        }
        parts.clear();
        rawResult = null;
    }    

    /**
     * Parses information from a raw html or xml string 
     * @param xmlString the raw input
     */
    public DCObject convertXmlToObject(String xmlString) {
        Software software = new Software();
        
        int indexBegin = 0;
        int indexEnd = 0;

        String productName = "";
        indexBegin = xmlString.indexOf("<ProductName>");
        if (indexBegin != -1) {
            indexBegin = indexBegin + 13;
            indexEnd = xmlString.indexOf("</ProductName>");
            productName = StringUtils.convertHttpString(xmlString.substring(indexBegin, indexEnd));
        }
        
        String amazonUrl = "";
        indexBegin = xmlString.indexOf("url=");
        if (indexBegin > -1) {
            indexBegin = indexBegin + 5;
            indexEnd = xmlString.indexOf(">") - 1;
            amazonUrl = xmlString.substring(indexBegin, indexEnd);
        }        

        String sID = "";
        if (amazonUrl.indexOf("ASIN/") > 0) {
        	sID = amazonUrl.substring(amazonUrl.indexOf("ASIN/") + 5, amazonUrl.length());
        	sID = sID.substring(0, sID.indexOf("/"));
        }
        
        indexBegin = xmlString.indexOf("<ImageUrlLarge>");
        if (indexBegin != -1) {
            try {
            	String quality = broadband ? "LZZZZZZZ.jpg" : "MZZZZZZZ.jpg";
                
                URL url = new URL("http://images.amazon.com/images/P/" + sID + ".01." + quality);
                HttpConnection connection = new HttpConnection(url);
                software.setValue(Software._J_PICTUREFRONT, connection.getImageAsBase64());
                connection.close();

            	url = new URL("http://images.amazon.com/images/P/" + sID + ".01.PT01." + quality);
                connection = new HttpConnection(url);
                software.setValue(Software._M_SCREENSHOTONE, connection.getImageAsBase64());
                connection.close();
                
                url =   new URL("http://images.amazon.com/images/P/" + sID + ".01.PT02." + quality);
                connection = new HttpConnection(url);
                software.setValue(Software._N_SCREENSHOTTWO, connection.getImageAsBase64());
                connection.close();

                url =   new URL("http://images.amazon.com/images/P/" +sID + ".01.PT03." + quality);
                connection = new HttpConnection(url);
                software.setValue(Software._O_SCREENSHOTTHREE, connection.getImageAsBase64());
                connection.close();
                
            } catch (Exception exp) {
                Messages.add(exp);
            }
        }        
        
        software.setValue(Software._A_NAME, StringUtils.convertHttpString(productName));
        software.setValue(Software._B_DESCRIPTION, getDescription(xmlString));
        software.setValue(Software._C_PUBLISHER, getManufacturer(xmlString));
        software.setValue(Software._D_PLATFORM, getPlatforms(xmlString));
        software.setValue(Software._E_RELEASEYEAR, getReleaseDate(xmlString));
        software.setValue(Software._F_WEBPAGE, amazonUrl); 
        software.setValue(Software._R_RATING, getRating(xmlString));

        return software;
    }

    private String getPlatforms(String rawResult) {
        int indexBegin = 0;
        int indexEnd = 0;
        
        String platforms = "";
        indexBegin = rawResult.indexOf("<Platforms>");
        if (indexBegin > -1) {
            indexBegin = indexBegin + 11;
            indexEnd = rawResult.indexOf("</Platforms>");
            platforms = rawResult.substring(indexBegin, indexEnd);

            String temp = "";
            indexBegin = platforms.indexOf("<Platform>");
            while(indexBegin > -1) {
                indexBegin = indexBegin + 10;
                indexEnd = platforms.indexOf("</Platform>");

                temp = temp + platforms.substring(indexBegin, indexEnd) + " ";

                indexEnd = indexEnd + 11;
                platforms = platforms.substring(indexEnd, platforms.length());
                indexBegin = platforms.indexOf("<Platform>");
            }
            platforms = temp;
        }
        return StringUtils.convertHttpString(platforms);
    }

    private Integer getRating(String rawResult) {
        int indexBegin = 0;
        int indexEnd = 0;
        
        String rating = "";
        indexBegin = rawResult.indexOf("<AvgCustomerRating>");
        if (indexBegin > -1) {
            indexBegin = indexBegin + 19;
            indexEnd = rawResult.indexOf("</AvgCustomerRating>");
            rating = rawResult.substring(indexBegin, indexEnd);
        }
        
        Integer iRating = new Integer(0);
        try { 
            if (rating.trim().length() > 0) {
            	iRating = new Integer(Math.round(Float.valueOf(rating).floatValue()));
            }
        } catch (Exception ignore) {};
        
        return iRating;
    }
    
    private String getManufacturer(String rawResult) {
        int indexBegin = 0;
        int indexEnd = 0;
        
        String manufacturer = "";
        indexBegin = rawResult.indexOf("<Manufacturer>");
        if (indexBegin > -1) {
            indexBegin = indexBegin + 14;
            indexEnd = rawResult.indexOf("</Manufacturer>");
            manufacturer = rawResult.substring(indexBegin, indexEnd);
        }
        return StringUtils.convertHttpString(manufacturer);
    }
    
    private Integer getReleaseDate(String rawResult) {
        int indexBegin = 0;
        int indexEnd = 0;
        
        String releaseDate = "";
        Integer releaseYear = new Integer(0);
        indexBegin = rawResult.indexOf("<ReleaseDate>");
        if (indexBegin != -1) {
            indexBegin = indexBegin + 13;
            indexEnd = rawResult.indexOf("</ReleaseDate>");
            releaseDate = rawResult.substring(indexBegin, indexEnd);
            try {
            	releaseDate = releaseDate.substring(releaseDate.indexOf(",") + 1, releaseDate.length()).trim();
                releaseYear = new Integer(Integer.parseInt(releaseDate));
            } catch (Exception ignore) {
            	//ignore.printStackTrace();
            }
        }
        return releaseYear;
    }
    
    /**
     * Creates a description out of several pieces of information
     * @param rawResult the raw input string
     */
    private String getDescription(String rawResult) {
        int indexBegin = 0;
        int indexEnd = 0;

        String companyDescription = "";
        indexBegin = rawResult.indexOf("<ProductDescription>");
        if (indexBegin != -1) {
            indexBegin = indexBegin + 20;
            indexEnd = rawResult.indexOf("</ProductDescription>");
            companyDescription = rawResult.substring(indexBegin, indexEnd);
        }

        String features = "";
        indexBegin = rawResult.indexOf("<Features>");
        if (indexBegin > -1) {
            indexBegin = indexBegin + 10;
            indexEnd = rawResult.indexOf("</Features>");
            features = rawResult.substring(indexBegin, indexEnd);

            String temp = "";
            indexBegin = features.indexOf("<Feature>");
            int counter = 1;
            while(indexBegin > -1) {
                indexBegin = indexBegin + 9;
                indexEnd = features.indexOf("</Feature>");

                temp += "" + (counter++) + ") " + features.substring(indexBegin, indexEnd) + "\n";

                indexEnd = indexEnd + 10;
                features = features.substring(indexEnd, features.length());
                indexBegin = features.indexOf("<Feature>");
            }
            
            if (temp.trim().length() > 0) {
            	features = "-- features --" + "\n" + temp;
            } else {
            	features = "";
            }
        }
        String userReviews = getUserReview(rawResult);
        
        String description = "";
        if (companyDescription.trim().length() > 0) {
            description += companyDescription;
        }
        
        if (userReviews.trim().length() > 0) {
            if (description.length() > 0) {
                description += "\n\n";
            }
            description += userReviews;
        }        
        
        if (features.trim().length() > 0) {
            if (description.length() > 0) {
            	description += "\n\n";
            }
            description += features;
        }
        
        return StringUtils.convertHttpString(description);
    }
    
    private String getUserReview(String xmlString) {
        int begin = xmlString.indexOf("<Reviews>");
        int end = xmlString.lastIndexOf("</Reviews>");
        
        String reviews = "";
        if (begin > -1 && end > -1) {
            String xmlReviews = xmlString.substring(begin + 16, end);
            
            int counter = 1;
            while (xmlReviews.indexOf("comment") > -1) {
                if (counter > 1) {
                	reviews += "\n\n";
                }
                begin = xmlReviews.indexOf("<Comment>");
                end = xmlReviews.indexOf("</Comment>"); 
                reviews += "-- review " + (counter++) + " --\n" + 
                           xmlReviews.substring(begin + 9, end);
                xmlReviews = xmlReviews.substring(end + 10, xmlReviews.length());
            }
        }
        return reviews;
    }
}
